#!/usr/bin/perl

# fwnight.pl
# run this every night to roll over firewall logs

# by eravin@panix.com (Ed Ravin), please send fixes or suggestions

# configurable things: adjust as appropriate to your system
###########################################################
$FWROOT="/winnt/fw";
$FWLOGDIR="$FWROOT/log";
$FWEXE="$FWROOT/bin/fw.exe";
$SENDMAIL="$FWROOT/bin/sendmail.exe";

$SAVEDIR="/fwlog";
$MESSAGES="$SAVEDIR/fwnight.out";
$FWADM='eravin@panix.com';
$MAILSERVER=166.84.1.66; # just in case

$LOGNAME="fwlog";
$MYLOG="$FWLOGDIR/$LOGNAME";

# NOTE: NT Posix commands won't accept DOS pathnames; use something
# better if you are copying between drives.
$POSIXROOT="c:/ntreskit/posix";
$CP="$POSIXROOT/cp.exe";

$FTPINFO='c:\local\bin\ftpme.txt';
$FTPHOST=nfs1;

$ARCHIVECMD="perl c:/local/bin/logmunch -t c:/local/bin/fwlogm.cfg";

################################

require "c:/local/bin/getopts.pl";

&Getopts('d');

$lastcommand="";

$DEBUG=0;
$DEBUG=$opt_d if defined($opt_d);

if (! $DEBUG)
{
    close STDOUT;
    open STDOUT, ">>$MESSAGES" || die "can't reopen STDOUT to $MESSAGES";
        
    close STDERR;
    open STDERR, ">>&STDOUT";
}

chdir $FWLOGDIR || &abort("chdir to $FWLOGDIR failed: $!");

($junk, $junk, $junk, $mday, $mon, $year, $junk, $junk, $junk)=
        localtime(time);

$year += 1900;
$today= sprintf "%04d.%02d.%02d", $year, $mon + 1, $mday;


# did last night's rollover have any problems?

if ( -f $MYLOG )
{ # ouch.  Rename the fwlog file so we don't lose it, and tell someone
        rename $MYLOG, "$MYLOG.overlooked.$today";
        warn "$MYLOG.overlooked.$today created from $MYLOG: help!";
}

if ( -s "$SAVEDIR/$LOGNAME" ) # should be empty
{
        # ouch : rename and tell someone
        rename "$SAVEDIR/$LOGNAME", "$SAVEDIR/$LOGNAME.overlooked.$today";
        warn "$SAVEDIR/$LOGNAME.overlooked.$today created due to non-zero size: help!";
}
else
{
        unlink "$SAVEDIR/$LOGNAME";     # trash zero-length bogus file
}

if ( -f $MESSAGES )
{
        rename $MESSAGES, "$MESSAGES.found.on.$today";
}

$todaylog="fwbl.$today";
$rc= &docommand("$FWEXE logswitch $todaylog");

$todaylog= "$todaylog.log"; # "fw logswitch" adds the suffix
if ( ! -f "$todaylog" )
{
        # ouch, tell someone
        $err= "failed command $rc: $lastcommand";
        &mailmsg($err);
        &abort($err);
}

$rc= &docommand("$FWEXE logexport -i $todaylog -o $MYLOG -n");
if ( ! -f $MYLOG )
{
        # ouch, tell someone
        $err="failed command: $rc: $lastcommand";
        &mailmsg($err);
        &abort($err);
}

$rc= &docommand("$CP $MYLOG $SAVEDIR/$LOGNAME");
if ( $rc != 0)
{
        warn "$CP $MYLOG $SAVEDIR/$LOGNAME returned $rc";
}

if ( -f "$SAVEDIR/$LOGNAME" )
{
        warn "$0: error unlinking $MYLOG: $!" unless unlink $MYLOG;
}
else
{
        warn "no resultfile: $lastcommand";
}

# drop the file off somewhere else for processing
chdir($SAVEDIR);
$rc= &docommand("ftp -s:$FTPINFO $FTPHOST");

# archive locally as well
$rc= &docommand("$ARCHIVECMD");
# note : logmuch will null out $SAVEDIR/$LOGNAME after copying & zip'ing

chdir $FWLOGDIR;
&findndel($FWLOGDIR, "^fwbl.*", 7);

#############exit code##############

if ( -s $MESSAGES )
{
        &abort("Error messages found: sending mail.");
}
#############end of main############

####################################
sub abort
{
        local ($errmsg)= @_;
        warn $errmsg;
        &mailmsg("<$MESSAGES");
        exit 3;
}

sub mailmsg
{
        local ($msg)= @_;
        local ($from, $to, $subject)= ("fwnight.pl", $FWADM, "FW log problem");

        if ( $msg =~ /^</ ) # input form filespec?
        {
        $msg =~ s,/,\\,g;
                &docommand("$SENDMAIL -s \"$subject\" -t $MAILSERVER -f \"$from\" $to $msg");
        }
        else
        {
                &docommand("echo $msg | $SENDMAIL -f \"$from\" -s \"$subject\" -t $to");
        }
}

sub docommand
{
    local ($cmd)= @_;

    $lastcommand= $cmd;
    print STDERR "execute: $cmd\n" if $DEBUG;
    return system $cmd;
}

sub findndel # directory, RE-to-match-filenames, days-old-to-kill
{
        local ($dir, $match, $expire)= @_;
        local ($file, $size, $atime, $mtime, $ctime, $age, $now);

        $now= time;
        opendir(FINDNDEL, $dir) || die "$0: findndel: $dir not found: $!";

        while ($file= readdir FINDNDEL)
        {
                next unless $file =~ /$match/i;
                ($junk, $junk, $junk, $junk, $junk, $junk, $junk, $size, $atime, $mtime,
$ctime, $junk, $junk)= stat("$dir/$file");

                $age= ($now - $mtime) / 60 / 60 / 24;
                if ($age > $expire)
                {
                        printf "findndel: file %s/%s is %d days old, delete\n",
                                $dir, $file, $age;
                        unlink "$dir/$file";
                }
        }
        closedir(FINDNDEL);
}